
function model_seasonal_var(lon,lat,cgps_location)

% Modeling deformation induced by seasonal variations of continental
% water in the Himalaya region : Sensitivity to Earth elastic structure
%
% Input: - lat: latitude vector of loading zone 
%        - lon: longitude vector of loading zone 
% (note that the GRACE loading grid has a 1x1 degree spacing)
%        - 'cgps_location': text file in between quotes with 
%                         station_name/longitude/latitude where you wish to
%                         compute displacements induced by surface loads
%
% Output: - grace_extract folder with equivalent water heigt files for 
%           loading zone / date
%         - green_functions_horizontal.mat if required (only once for a specific zone)
%         - green_functions_vertical.mat if required (only once for a specific zone)
%         - disp_zone folder with .mat files of east,north and
%            vertical displacements for all points in loading zone, by
%            date
%         - disp_gps folder with ascii files of t/east/north/vertical
%            displacements at each location referenced in 'cgps_location' 
%            file     
% Calls: subfunctions within file
% Called by: main.m
%
% K. Chanard, last modified 20-03-14, from M. Barr 31-07-14


%% load GRACE data
grace_directory = dir(fullfile('./grace_extract/20*'));
grace_files = {grace_directory.name};
ngrace=length(grace_files);

time = textread('./calendar.txt','%f');

%% GPS coordinates
[rname,rlon,rlat] = textread(cgps_location,'%s%f%f'); 
ngps = length(rname);

%% BUILD THE MESH
[iclon,iclat,~] = textread('./grace_extract/2002_08_08_2002_08_17','%f%f%f');
deg2rad = pi/180;
nlat = length(lat);
nlon = length(lon);
nall = nlat*nlon;
iclon = reshape(iclon,nlon,nlat);         % return a nlon-by-nlat matrix whose elements are taken column-wise from iclon
iclat = reshape(iclat,nlon,nlat);
iwlon = ones(size(iclat(:)));
iwlat = ones(size(iclat(:)));

[oolon,oolat] = meshgrid(lon,lat);        % replicates the grid vecor lon and lat to produce a full grid

%% COMPUTE THE GREEN FUNCTIONS

prompt = 'Do you want to compute Greens function? y/n :  ';
str = input(prompt,'s');

if str=='y'
%   Vertical component

    for n=1 : size(oolat(:))
        disp(sprintf('Computing vertical Green functions at grid point # %d out of %d',n,nall)) 
        olat = oolat(n).*deg2rad;
        olon = oolon(n).*deg2rad;
        [udisp] = load_disp_u(iclat.*deg2rad,iclon.*deg2rad,iwlat.*deg2rad,iwlon.*deg2rad,olat,olon,1);
        greenf_u(n,:) = udisp(:);
    end

    save green_functions_vertical greenf_u

%    Horizontal components

    for n=1 : size(oolat(:))
        disp(sprintf('Computing horizontal Green functions at grid point # %d out of %d',n,nall)) 
        olat = oolat(n).*deg2rad;
        olon = oolon(n).*deg2rad;
        [edisp,ndisp] = load_disp(iclat.*deg2rad,iclon.*deg2rad,iwlat.*deg2rad,iwlon.*deg2rad,olat,olon,2);
        greenf_e(n,:) = edisp(:);
        greenf_n(n,:) = ndisp(:);
    end

    save green_functions_horizontal greenf_e greenf_n

end

    load green_functions_horizontal
    load green_functions_vertical

    if ~exist('disp_zone', 'dir')
      mkdir('disp_zone');
    end   
    for ii=1:ngrace

        file = fullfile('./grace_extract',grace_files{ii}); 
        [iclon,iclat,loads] = textread(file,'%f%f%f');
        tloads(ii,:)=loads;
        loads=loads./1e3; % mm to m of equivalent water height
        iclon = reshape(iclon,nlon,nlat);
        iclat = reshape(iclat,nlon,nlat);

        name_grace_file = grace_files(ii);

        for n=1 : size(oolat(:))
            ccn(ii,n) = nansum(greenf_n(n,:).*loads(:)'); % (ii,n) ii and n mean temporal and spatial, respectively.
            cce(ii,n) = nansum(greenf_e(n,:).*loads(:)');
            ccu(ii,n) = nansum(greenf_u(n,:).*loads(:)');

        end

        tmpn = reshape(squeeze(ccn(ii,:)),nlat,nlon);     % return the nlat-by-nlon matrix tmpn whose elements are taken column-wise from squeeze(ccn(ii,:))
        tmpe = reshape(squeeze(cce(ii,:)),nlat,nlon);
        tmpu = reshape(squeeze(ccu(ii,:)),nlat,nlon);

        alln(ii,:,:) = tmpn;                              % alln alle allu are all displacements on the grid
        alle(ii,:,:) = tmpe;
        allu(ii,:,:) = tmpu;

    
        file_du = sprintf('disp_zone/%s_du.mat',grace_files{ii});
        save(file_du,'tmpu');
        file_dn = sprintf('disp_zone/%s_dn.mat',grace_files{ii});
        save(file_dn,'tmpn');
        file_de = sprintf('disp_zone/%s_de.mat',grace_files{ii});
        save(file_de,'tmpe');



        for jj=1 : ngps
            ein = interp2(oolon,oolat,tmpn,rlon(jj),rlat(jj));
            eie = interp2(oolon,oolat,tmpe,rlon(jj),rlat(jj));
            eiu = interp2(oolon,oolat,tmpu,rlon(jj),rlat(jj));
            nn(ii,jj) = ein;
            ee(ii,jj) = eie;
            uu(ii,jj) = eiu;
                                                          % We do an interpolation to obtaint nn ee uu; Their are
                                                          % displacements at each c-gps station over time;
        end



    end

    save('grace_extract/grace_ts','time','tloads','iclon','iclat');  % save displacements at cgps stations in .mat file

                   % displacements (time,gps_station)

    if ~exist('disp_gps', 'dir')
      mkdir('disp_gps');
    end               

    save('disp_gps/displacements_cgps','time','uu','nn','ee','rname','ngps');  % save displacements at cgps stations in .mat file
        for i=1:ngps
           tmp=[time,nn(:,i),ee(:,i),uu(:,i)];
           file_ts=sprintf('disp_gps/%s.txt',rname{i}); % save time series of displacements at each cgps station in text file
           save(file_ts,'-ascii','tmp')   
        end
    
end

function [udisp] = load_disp_u(iclat,iclon,iwlat,iwlon,olat,olon,indx)
% green functions for prem with continental crust based on Guo et al.(2004)
% indx 1:ur 2:utheta 3:G^E 4:t^E 5:err 6:ethetatheta 7:elamdalamda
% indx 3 and  x by 1E5.

readf('prem.data');
nlen = length(iwlat);
disp_u = zeros(nlen,1);

for n=1 : nlen
    wlat = iwlat(n); 
    wlon = iwlon(n);
    clat = iclat(n); 
    clon = iclon(n);
    
    if indx == 1
        udisp(n) = gdisp2(clat,clon,wlat,wlon,olat,olon,indx);
    end
    
    if indx == 2
        [displ(n)] = gdisp2(clat,clon,wlat,wlon,olat,olon,indx);
        [az] = azrad(olat,olon,clat,clon);
        ndisp(n) = displ(n).*cos(az);
        edisp(n) = displ(n).*sin(az);
    end
    
end
end

function [edisp, ndisp] = load_disp(iclat,iclon,iwlat,iwlon,olat,olon,indx)
% green functions for prem with continental crust based on Guo et al.(2004)
% indx 1:ur 2:utheta 3:G^E 4:t^E 5:err 6:ethetatheta 7:elamdalamda
% indx 3 and  x by 1E5.

readf('prem.data');
nlen = length(iwlat);
disp_u = zeros(nlen,1);

for n=1 : nlen
    wlat = iwlat(n);
    wlon = iwlon(n);
    clat = iclat(n); 
    clon = iclon(n);
    
    if indx == 1
        udisp(n) = gdisp2(clat,clon,wlat,wlon,olat,olon,indx);
    end
    
    if indx == 2
        [disp(n)] = gdisp2(clat,clon,wlat,wlon,olat,olon,indx);
        [az] = azrad(olat,olon,clat,clon);
        ndisp(n) = disp(n).*cos(az);
        edisp(n) = disp(n).*sin(az);
    end
    
end
end


function [gauss] = gdisp2(clat,clon,wlat,wlon,olat,olon,indx)
% Gauss-Legendre Quadrature

earthr = 6.371e6;
gausst = 9999;
[area] = areasph(clat,clon,wlat,wlon);

for dim=1 : 6;
    r = [];
    w = [];
    
    if (dim==1);
        r(1: 2) = [-0.577350269189626; 0.577350269189626];
        w(1: 2) = [ 1.000000000000000; 1.000000000000000];
    elseif (dim==2);
        r(1: 3) = [-0.774596669241483; 0.000000000000000; 0.774596669241483];
        w(1: 3) = [ 0.555555555555556; 0.888888888888889; 0.555555555555556];
    elseif (dim==3);
        r(1: 4) = [-0.861136311594053;-0.339981043584856; 0.339981043584856; 0.861136311594053];
        w(1: 4) = [ 0.347854845137454; 0.652145154862546; 0.652145154862546; 0.347854845137454];
    elseif (dim==4);
        r(1: 5) = [-0.906179845938664;-0.538469310105683; 0.000000000000000; 0.538469310105683; 0.906179845938664];
        w(1: 5) = [ 0.236926885056189; 0.478628670499367; 0.568888888888889; 0.478628670499367; 0.236926885056189];
    elseif (dim==5);
        r(1: 5) = [-0.973906528517;-0.865063366689;-0.679409568299;-0.433395394129;-0.14887433898]; r(6:10)=-fliplr(r(1:5));
        w(1: 5) = [ 0.066671344309; 0.149451349151; 0.219086362516; 0.269266719310; 0.29552422472]; w(6:10)= fliplr(w(1:5));
    elseif (dim==6);
        r(1:10) = [-0.993128599185;-0.963971927278;-0.912234428251;-0.839116971822;-0.74633190646;...
            -0.636053680727;-0.510867001951;-0.373706088715;-0.227785851142;-0.07652652113]; r(11:20)=-fliplr(r(1:10));
        w(1:10)=[ 0.017614007068; 0.040601429882; 0.062672048298; 0.083276741551; 0.10193011983;...
            0.118194531969; 0.131688638458; 0.142096109327; 0.149172986482; 0.15275338714]; w(11:20)= fliplr(w(1:10));
    end
    
    ww = w'*w;
    rlat = clat+.5.*wlat.*r;
    rlon = clon+.5.*wlon.*r;
    
    [rrlat,rrlon] = meshgrid(rlat,rlon);
    angp = acos(sin(rrlat(:)).*sin(olat)+cos(rrlat(:)).*cos(olat).*cos(rrlon(:)-olon));
    [grnp] = greenp(angp,indx);
    gauss = sum(ww(:).*grnp(:)./(angp(:).*earthr)).*1E-12.*area./4;
    
    if abs(1-gausst./gauss) < 0.0001; break; end; gausst=gauss;
end
end



function [area] = areasph(cclat,cclon,wlat,wlon)
% GRACE data are date on each 1x1 grid of the earth. THis function compute
%the area of each 1x1 compartement; It depends on where we are on the
%earth (latitude, pole)

earthr = 6.371e6;

if wlat*wlon > 3e-5 ;
    alat = cclat-0.5.*wlat; alon = cclon-0.5.*wlon;
    blat = cclat+0.5.*wlat; blon = cclon-0.5.*wlon;
    clat = cclat+0.5.*wlat; clon = cclon+0.5.*wlon;
    dlat = cclat-0.5.*wlat; dlon = cclon+0.5.*wlon;
    
    [sinab,cosab] = angrad(alat,alon,blat,blon);
    [sinbc,cosbc] = angrad(blat,blon,clat,clon);
    [sincd,coscd] = angrad(clat,clon,dlat,dlon);
    [sinac,cosac] = angrad(alat,alon,clat,clon);
    [sinbd,cosbd] = angrad(blat,blon,dlat,dlon);
    [sinda,cosda] = angrad(dlat,dlon,alat,alon);
    
    a = acos((cosbd-cosab.*cosda)./(sinab*sinda));
    b = acos((cosac-cosab.*cosbc)./(sinab*sinbc));
    c = acos((cosbd-coscd.*cosbc)./(sincd*sinbc));
    d = acos((cosac-coscd.*cosda)./(sincd*sinda));
    
    area = (a+b+c+d-2*pi).*earthr.*earthr;
    
    for i=1 : length(cclat)
        
        if area(i)==0 || isnan(area(i));
            area(i) = wlat(i).*wlon(i).*earthr.*earthr.*cos(cclat(i)).*cos(0.5*wlat(i));
        end;
        
    end
else
    area = wlat.*wlon.*earthr.*earthr.*cos(cclat).*cos(0.5*wlat);
end
end

function [sinang,cosang] = angrad(alat,alon,blat,blon)
cosang = sin(alat).*sin(blat)+cos(alat).*cos(blat).*cos(alon-blon);
sinang = sqrt(1-cosang.*cosang);
end


function readf(grnfile)
% This function extract prem.data. Theses data are commpute for a spherical
% perfectly elastic Earth Model based on load Love numbers theory.

global ang grn
deg2rad = pi/180;
tmp = reshape(dlmread(grnfile,''),50,8);
grn = tmp(:,2:8);
ang = tmp(:,1).*deg2rad;
end

function [grnp] = greenp(angp,indx)
global ang grn
n = length(angp);
m = length(ang);
mind = (1:m)'; mind = mind(:,ones(1,n));
nind = (1:n)'; nind = nind(:,ones(1,m));
dang = ang(mind);
dgrn = grn(:,indx); dgrn = dgrn(mind);
[tmp,sind] = sort(abs(dang-angp(nind)'));
tmp = dang(sind); x(1:3,:) = tmp(1:3,:);
tmp = dgrn(sind); y(1:3,:) = tmp(1:3,:);
[grnp] = quadr(x,y,angp');
end

function [yp] = quadr(x,y,xp)
xa = x(2,:)-x(1,:);
ya = y(2,:)-y(1,:);
xb = x(3,:)-x(2,:);
yb = y(3,:)-y(2,:);
xc = x(1,:)-x(3,:);
xxc = 1.0./(xa.*xb.*xc);
a = -(xa.*yb-xb.*ya).*xxc;
b = -(ya.*xb.*(x(2,:)+x(3,:)) -...
    yb.*xa.*(x(1,:)+x(2,:))).*xxc;
c = -(y(1,:).*x(2,:).*x(3,:).*xb +...
    x(1,:).*y(2,:).*x(3,:).*xc +...
    x(1,:).*x(2,:).*y(3,:).*xa).*xxc;
yp = xp.*(a.*xp + b) + c;
end


function [az] = azrad(alat,alon,blat,blon)
%Compute azimut of a point in rad

az = atan2(cos(alat).*sin(alon-blon),cos(blat).*sin(alat)-sin(blat).*cos(alat).*cos(alon-blon));

end

